/*
 * rpcd - UBUS RPC server
 *
 *   Copyright (C) 2013-2014 Jo-Philipp Wich <jow@openwrt.org>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef __RPC_UCI_H
#define __RPC_UCI_H

#include <glob.h>
#include <limits.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <inttypes.h>
#include <libubus.h>
#include <uci.h>

#define RPC_UCI_DIR_PREFIX	"/var/run/rpcd"
#define RPC_UCI_SAVEDIR_PREFIX	RPC_UCI_DIR_PREFIX "/uci-"
#define RPC_SNAPSHOT_FILES	RPC_UCI_DIR_PREFIX "/snapshot-files/"
#define RPC_SNAPSHOT_DELTA	RPC_UCI_DIR_PREFIX "/snapshot-delta/"
#define RPC_UCI_DIR		"/etc/config/"
#define RPC_APPLY_TIMEOUT	60

int rpc_uci_api_init(struct ubus_context *ctx);

void rpc_uci_purge_savedirs(void);

struct uci_granular_ctx_s *rpc_uci_load_granular(const char *username,
		const char *config_name, const char *section_name,
		const char *section_type, const char *sid);
int rpc_uci_check_granular_is_allowed(const char *option);
struct blob_buf *rpc_uci_get_glob_bb(void);
int rpc_uci_dump_package(struct uci_package *p, const char *name,
		struct blob_attr *type, struct blob_attr *matches, const char *username,
		const char *sid);
int rpc_uci_dump_section(struct uci_section *s, const char *name, int index, const char *username);
int rpc_uci_dump_option(struct uci_option *o, const char *name);
int rpc_uci_init_uci(void);
void rpc_uci_commit_cursor(struct uci_package *p);
int rpc_uci_set(struct ubus_context *ctx, struct ubus_object *obj,
            struct ubus_request_data *req, const char *method,
            struct blob_attr *msg);
int rpc_uci_revert_commit(struct ubus_context *ctx, struct blob_attr *msg,
		bool commit);
int rpc_uci_add(struct ubus_context *ctx, struct ubus_object *obj,
            struct ubus_request_data *req, const char *method,
            struct blob_attr *msg);
int rpc_uci_delete(struct ubus_context *ctx, struct ubus_object *obj,
               struct ubus_request_data *req, const char *method,
               struct blob_attr *msg);
int rpc_uci_rename(struct ubus_context *ctx, struct ubus_object *obj,
               struct ubus_request_data *req, const char *method,
               struct blob_attr *msg);
void rpc_uci_clean(void);
#endif
